package AbsenceSchedules;

import InitialSchedules.ModelInitialSchedules;
import ScheduleUtil.SchedulesArray;
import jdistlib.rng.RandomEngine;
import org.chocosolver.solver.Solution;

public class AbsenceSchedulesArray extends SchedulesArray{
    
    protected boolean[][] partTimeEmployeeUnavailabilitySchedules;
    protected boolean[][] fullTimeEmployeeUnavailabilitySchedules;
    
    public AbsenceSchedulesArray(SchedulesArray myScheduleArray) {
        super(myScheduleArray);
    } 
    
    public AbsenceSchedulesArray(AbsenceSchedulesArray myAbsenceScheduleArray) {
        super(myAbsenceScheduleArray);
        this.partTimeEmployeeUnavailabilitySchedules = new boolean[this.maxPartTimeEmployee][this.workPeriodsPerSchedule];
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
                this.partTimeEmployeeUnavailabilitySchedules[employee][workPeriod] = myAbsenceScheduleArray.partTimeEmployeeUnavailabilitySchedules[employee][workPeriod];
            }
        }
        this.fullTimeEmployeeUnavailabilitySchedules = new boolean[this.maxFullTimeEmployee][this.workPeriodsPerSchedule];
        for (int workPeriod = 0; workPeriod < this.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
                this.fullTimeEmployeeUnavailabilitySchedules[employee][workPeriod] = myAbsenceScheduleArray.fullTimeEmployeeUnavailabilitySchedules[employee][workPeriod];
            }
        }
    }
    
    public void generateAbsences(RandomEngine r) {
        this.partTimeEmployeeUnavailabilitySchedules = new boolean[this.maxPartTimeEmployee][this.workPeriodsPerSchedule];
        for (int i = 0; i < this.maxPartTimeEmployee; i++) {
            boolean[] a = new AbsencesVector(this.workPeriodsPerSchedule, r).getAbsencesVector();
            for (int j = 0; j < this.workPeriodsPerSchedule; j++) {
                this.partTimeSchedules[i][j] = this.initialPartTimeSchedules[i][j] && a[j];
            }
        }
        this.fullTimeEmployeeUnavailabilitySchedules = new boolean[this.maxFullTimeEmployee][this.workPeriodsPerSchedule];
        for (int i = 0; i < this.maxFullTimeEmployee; i++) {
            boolean[] a = new AbsencesVector(this.workPeriodsPerSchedule, r).getAbsencesVector();
            for (int j = 0; j < this.workPeriodsPerSchedule; j++) {
                this.fullTimeSchedules[i][j] = this.fullTimeSchedules[i][j] && a[j];
            }
        }
        updateAllEmployeesPerWorkPeriod();
        updateAllWorkingPeriodsPerPartTimeEmployees();
        updateAllWorkingPeriodsPerFullTimeEmployees();
    }
    
    
    protected boolean isPartTimeEmployeeAbsent(int employee, int workPeriod) {
        return this.partTimeSchedules[employee][workPeriod] != this.initialPartTimeSchedules[employee][workPeriod];
    }
    
    protected boolean isFullTimeEmployeeAbsent(int employee, int workPeriod) {
        return this.fullTimeSchedules[employee][workPeriod] != this.initialFullTimeSchedules[employee][workPeriod];
    }
    
}